/***************************************************************************
                          rcfile.c
  description: persistant game options
                             -------------------
    begin                : Wed April 26 2010
    date                 : 26/04/2010
    copyright            : (C) 2001 Interlinux Ltd
    email                : dbrasher@interlinux.co.uk
    contributed by       : David Jensen
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <ctype.h>

#include "config.h"
#include "sound.h"
#include "data.h"  // DIR_SEP, PATH_MAX

#define MAX_LINE_LEN  256


typedef enum {
	D_ERR = -2,     /*Does not exist, or other error*/
	D_NOENT = -1,   /*Does not exist*/
	D_DIR = 0,      /*OK, Exists and is a directory*/
	D_NOT_DIR = 1   /*Exists, not a directory*/
} DirVal;

/* config file strings*/
char *rc_dirname = NULL;
char *rc_filename = NULL;

/* local prototypes */
static DirVal   _dsi_isdir(const char *path);
static DirVal   _dsi_ckdir(const char *path, int create_dir);
static char    *_dsi_get_tag_string(FILE *f, char *tag);


static int _dsi_cp_sysrc(const char *src, const char *dest)
{
	char buf[PATH_MAX];
	FILE *sf, *df;
	size_t bytes_read;

	sf = fopen(src, "r");
	if (sf == NULL)
		return -1;
	df = fopen(dest, "w");
	if (df != NULL)
	{
		while (!feof (sf))
		{
			bytes_read = fread(buf, 1, PATH_MAX, sf);
			fwrite(buf, 1, bytes_read, df);
		
		}
		fclose(df);
	}
	fclose(sf);
	return 0;
}

/**
 * Get user configuration directory, attemp xdg compliance.
 *
 * Directory creation attempted when non-existing, if create_dir = 1.
 * dsi_rc_init();
 * @param: rcname       base name of the rcfile.
 * @param: create_dir   boolean, 0 don't create, 1 create.
 * Returns:             rc_filename or NULL,
 *                      rc_dirname and rc_filename allocated or NULL.
 */
char *dsi_rc_init(char *rcname, int create_dir)
{
	char *home;
	char buf[PATH_MAX];
	int err = D_ERR;
	size_t len;
	int dot = 0;
	int n = 0;

	buf[0] = '\0';
	home = getenv("XDG_CONFIG_HOME");
	if (home)
	{
		err = _dsi_ckdir(home, create_dir);
		if (err == D_DIR)
		{
			n = snprintf(buf, PATH_MAX, "%s%s%s", home,
					DIR_SEP, PACKAGE_NAME);
			buf[PATH_MAX - 1] = '\0';
			err = _dsi_ckdir(buf, create_dir);
		}
	}
	if (err != D_DIR)
	{
		home = getenv("HOME");
		if (home)
		{
			err = _dsi_isdir(home);
			if (err != D_DIR)
				return NULL;		/*Bailout: can't get home dir*/

			/* try xdg compliance */
			n = snprintf(buf, PATH_MAX, "%s%s%s", home,
					DIR_SEP, ".config");
			buf[PATH_MAX - 1] = '\0';
			err = _dsi_ckdir(buf, create_dir);

			if (err != D_DIR)	/* $HOME fallback */
			{
				n = snprintf(buf, PATH_MAX, "%s", home);
				buf[PATH_MAX - 1] = '\0';
				dot = 1;	/* use '.' in home */
			}
		}
	}
	if (n > 0)	/*we have a config dir */
	{
		len = n + strlen(PACKAGE_NAME) + 2 + dot;
		if (len < (PATH_MAX - 1))
		{
			rc_dirname = malloc(len);
			if (!rc_dirname)
				return NULL;		/*Bailout: allocation error*/
			sprintf(rc_dirname, "%s%s%s%s", buf, DIR_SEP,
				dot ? "." : "", PACKAGE_NAME);
			err = _dsi_ckdir(rc_dirname, create_dir);
			if (err != D_DIR)
				goto errout1;
			len += strlen(rcname) + 1;
			if (len > (PATH_MAX - 1))
				goto errout1;
			rc_filename = malloc(len);
			if (!rc_filename)
				goto errout1;
			sprintf(rc_filename, "%s%s%s", rc_dirname, DIR_SEP,
					rcname);
			err = _dsi_isdir(rc_filename);
			if (err == D_NOENT)
			{
				snprintf(buf, PATH_MAX, "%s%s%s", 
						DSI_DIR, DIR_SEP, rcname);
				_dsi_cp_sysrc(buf, rc_filename);
			}
		}
	}
	return rc_filename;
errout1:
	free(rc_dirname);
	rc_dirname = NULL;
	return NULL;
}


/**
 * dsi_get_tag_arg()
 * @param: f    file handle.
 * @param: tag  string to search for in the configuration file.
 * Returns:     if tag and = is found, the string following the =.
 */
char *dsi_get_tag_arg(FILE *f, char *tag)
{
	char *s;
	char *e;

	if ((s = _dsi_get_tag_string(f, tag)) != NULL)
	{
		e = strchr(s, '=');
		if (e)
			++e;
		{
			s = e;
			/* strip line endings: isspace requires c99? */
			while (*s && !isspace(*s))
				++s;
			*s = '\0';
			if (e[0])
				return e;
		}
	}
	return NULL;
}


/**
 * dsi_open_prefs()
 * @param:  filename    rcfile filename to read.
 * @param:  groupname   group to read. Note! this is not a full implementation.
 *                      At this point, group just indicates it is the correct
 *                      rcfile.
 * Returns: an open file handle if the rcfile is valid. otherwise NULL.
 */
FILE *dsi_open_prefs(char *filename, char *groupname)
{
	FILE *f = NULL;

	if (filename == NULL || groupname == NULL)
		return NULL;

	f = fopen(filename, "r");
	if (f == NULL)
		return NULL;

	if (_dsi_get_tag_string(f, groupname) == NULL)
	{
		fclose(f);
		printf("%s: invalid rcfile '%s'\n", PACKAGE_NAME, filename);
		return NULL;
	}
	return f;
}


/**** local funcs ********************************************/

/**
 * _dsi_get_tag_string()
 * @param: f    file handle.
 * @param: tag  string to search for in the configuration file.
 * Returns:     the tag and value as a string, or NULL.
 */
static char *_dsi_get_tag_string(FILE *f, char *tag)
{
	static char buf[MAX_LINE_LEN];
	char *ret = NULL;

	if (f == NULL)
		return ret;

	buf[0] = '\0';
	if (fseek(f, 0, SEEK_SET) != 0)
		return ret;
	while (!feof (f))
	{
		if (fgets(buf, MAX_LINE_LEN, f))
		{
			if (!strncmp(tag, buf, strlen (tag)))
			{
				ret = buf;
				break;
			}
		}
	}
	return ret;
}


/**
 * _dsi_isdir()
 * is path a directory?
 *
 * @path:       pathname to test.
 *
 * Returns:     See DirVal enum top of file.
**/
static DirVal _dsi_isdir(const char *path)
{
	struct stat sinfo;

	if (path) {
		if ( (stat (path, &sinfo)) == 0 ) {
			if (!S_ISDIR(sinfo.st_mode))
				return D_NOT_DIR;
			return D_DIR;
		}
	}
	return D_NOENT;
}


/**
 * _dsi_ckdir()
 * tests a directory, ceates if possible.
 *
 * @path:       pathname to test.
 * @create_dir: boolean, 0 don't create, 1 create.
 *
 * Returns:     See DirVal enum top of file.
**/
static DirVal _dsi_ckdir(const char *path, int create_dir)
{
	int err = D_ERR;

	if (path)
	{
		err = _dsi_isdir(path);
		if (err == D_NOENT && create_dir == 1)
		{

			#ifdef __LINUX__
            err = mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR);
            #endif
			if (err != D_DIR)
				err = D_ERR;
		}
	}
	return err;
}
